import { MigrationInterface, QueryRunner, Table, TableForeignKey } from "typeorm";

export class RolePermission1724771491085 implements MigrationInterface {
  name = "RolePermission1724771491085";

  public async up(queryRunner: QueryRunner): Promise<void> {
    await queryRunner.createTable(
      new Table({
        name: "role",
        columns: [
          {
            name: "id",
            type: "int",
            isPrimary: true,
            isNullable: false,
            isGenerated: true,
            generationStrategy: "increment",
          },
          {
            name: "role",
            type: "varchar",
            length: "255",
            isNullable: false,
          },
        ],
      }),
      true
    );

    await queryRunner.createTable(
      new Table({
        name: "role_permission",
        columns: [
          {
            name: "permission",
            type: "varchar",
            length: "255",
            isPrimary: true,
            isNullable: false,
          },
          {
            name: "roleId",
            type: "int",
            isPrimary: true,
            isNullable: false,
          },
        ],
      }),
      true
    );

    await queryRunner.renameTable("permission", "user_permission");

    await queryRunner.createTable(
      new Table({
        name: "user_roles",
        columns: [
          {
            name: "userId",
            type: "int",
            isPrimary: true,
            isNullable: false,
          },
          {
            name: "roleId",
            type: "int",
            isPrimary: true,
            isNullable: false,
          },
        ],
      }),
      true
    );

    await queryRunner.createForeignKey(
      "role_permission",
      new TableForeignKey({
        columnNames: ["roleId"],
        referencedColumnNames: ["id"],
        referencedTableName: "role",
        onDelete: "CASCADE",
      })
    );

    await queryRunner.createForeignKey(
      "user_roles",
      new TableForeignKey({
        columnNames: ["userId"],
        referencedColumnNames: ["id"],
        referencedTableName: "user",
        onDelete: "CASCADE",
      })
    );

    await queryRunner.createForeignKey(
      "user_roles",
      new TableForeignKey({
        columnNames: ["roleId"],
        referencedColumnNames: ["id"],
        referencedTableName: "role",
        onDelete: "CASCADE",
      })
    );
  }

  public async down(queryRunner: QueryRunner): Promise<void> {
    const user_roles = await queryRunner.getTable("user_roles");
    const roles_foreignKey = user_roles.foreignKeys.find((fk) => fk.columnNames.indexOf("roleId") !== -1);
    const user_foreignKey = user_roles.foreignKeys.find((fk) => fk.columnNames.indexOf("userId") !== -1);
    await queryRunner.dropForeignKey("user_roles", roles_foreignKey);
    await queryRunner.dropForeignKey("user_roles", user_foreignKey);
    await queryRunner.dropTable("user_roles");

    const role_permission = await queryRunner.getTable("role_permission");
    const permission_foreignKey = role_permission.foreignKeys.find((fk) => fk.columnNames.indexOf("roleId") !== -1);
    await queryRunner.dropForeignKey("role_permission", permission_foreignKey);
    await queryRunner.dropTable("role_permission");

    await queryRunner.dropTable("role");

    await queryRunner.renameTable("user_permission", "permission");
  }
}
